# Implementation

MagPen is composed of two main areas, the hardware and the software. The hardware consists of a Pen/Electromagnet and the software consists of an Android application with various algorithms to map the magnetometer values onto an x-y plane.


Hardware: Pen/Electromagnet
Software: Android Application
Magnetometer Sensor Research
Selection of Various Pen Attributes

**Hardware**

Early on in the project, we simply used a strong magnet to understand the magnetometer readings and the various magnetic fields. Once we got a strong hold on the magnetometer, we were able to conduct research into electromagnets. The use of electromagnets is necessary in this project to allow the magnetic field to turn “on” and “off”. This enables us to detect when the pen is actually writing and when it is hovering in the air.

Although we were unable to integrate this with a pen, we were able to create an electromagnet in the shape of a pen (shown above). This pen consists of a long rod with wires coiled around it, essentially an electromagnet. Additional research went into this to determine what kind of electromagnet would work best. Should we use a long/short rod, bare/insulated wire, and should the battery have higher/lower voltage? From our findings, using a long rod, with larger battery voltage produces the strongest magnetic field. The stronger the magnetic field, the stronger the readings of the magnetometer. The type of wire (bare or insulated) did not have any affect on the strength of the magnetic field. 

Since we were unable to integrate this into an actual pen, we could not successfully detect when the tip was pressed down. Therefore to simulate the interaction of the tip pressed down, a button was added on the pen. This button connects the entire circuit together thus allowing a magnetic field to be emitted. The MagPen application can detect when the button is clicked or double clicked and execute an action. Future work would include detecting when the tip has been pressed down and using those readings for the pen input. Thus freeing the button on the pen to be mapped to other actions such as start or stop recording audio. 

**Click Detection Algorithm**

As stated before, it is possible to detect a single click or double click by monitoring the magnetic sensor readings. Currently only double click is enabled in the application as it proves output lower amount of false positives. The algorithm takes in a list of magnetometer readings (e.g. last 10 readings) and calculates the differences between the values. Normally, if a user is not clicking anything, there should be minimal difference in the values (all between -3 and 3). However when the user clicks the pen, the pen will emit a magnetic field for a few seconds. Therefore, there will be a quick jump up in the magnetometer readings and a drop in magnetometer readings after a few seconds. We can detect this by checking for any jumps in the readings. 

To detect double click, we detect a baseline value below a certain threshold, a jump in the values, a baseline, a jump again in the values and a baseline. To detect a single click, we check for a baseline value, a jump in the values and a baseline again. However, detecting single click had a larger amount of false positives. The algorithm detected the pen moving away as a single click. Therefore currently only double click is enabled in the application. 

**Software**

The android application allows the user to create, view, and save documents. There is also the option to view raw magnetometer sensor readings and change application settings (e.g the algorithm). 

The sensor readings screen will display the raw magnetometer values in the x,y,z planes as well as intensity. These values are plotted on a continuously updating line graph (bottom left image). For debugging purposes, the user can calibrate the 4 corners to see the values mapped out on an x-y plane. Once the corners are calibrated, and the Readings option is deselected, a circle will be mapped on to an xy plane and will as the magnet is moved around.

In the settings screen, the user has the option to select various algorithms that map the magnetometer sensor readings onto the xy plane. These algorithms are described in detail later on in this document.

From the main screen, the user can tap the new document icon to get to the New Document Screen. Again similar to the sensors readings screen, the user will need to calibrate the 4 corners before using the magnet or magnet based pen (MagPen) as stroke inputs. This is done by tapping through the calibrate buttons on the bottom. 

The user also has the ability to draw with touch along with magnets. This allows for quick annotation of the document while writing with the MagPen. To start magnet input, the user can double click on the MagPen or tap the rightmost pen icon. When pen input is enabled, the pen will turn white to display that magnetic input is being accepted. The other paint brush and color picker icon allows the user to choose the stroke size and color. This is described in more detail later on in this document as well.

The save icon will save the notes/drawings locally on the device. If needed the user can revisit the home screen to see the saved image.

**Magnetometer Sensor Algorithms**

This project required a heavy research component to map the magnetometer readings to an x-y plane. Three different algorithms have been implemented and are selectable from the settings menu. Before any magnetic inputs are used the pen must first be calibrated. This is done by pressing the calibration button at the bottom of the screen and following the instruction. Essentially it asks the user to place the pen at the four corners of their working surface and press the button again. This gets a reading for the magnetic field when the pen is at each corner, so the application can use these as references for locating the current position of the pen. 

The first algorithm is a rather simple algorithm that uses only the ‘x’ and ‘y’ axis readings from the magnetometer. In the application we have called this ‘Four Screen Corners ’. This simply takes the current readings and places the ‘x’ value within the minimum and maximum ‘x’ values from the calibrated references, then does the same for the ‘y’ values. In order to try and compensate for inconsistent values, for example a ‘y’ value that is slightly higher at the bottom left than bottom right, the ‘y’ value is calculated using the top-left and bottom-left references, and then the top-right and bottom-right, then averages these values.The same is done for the ‘x’ value but using the the two top references and then the two bottom ones. 

The second algorithm attempts to use two dimensional trilateration in order to locate the pen in relation to the reference points. In the application this is called ‘Trilateration’. This algorithm takes the difference in field strength between the current position and three of the reference points. It uses these differences as pseudo distances and acts as though a circle is formed around each of the reference points whose radius is this distance. It then calculates the intersection points of these three circles to find the current location of the pen in relation to the reference points. 

The third algorithm attempts to change the frame of reference for the readings to zero out the ‘z’ values then projects these points onto the screen. In the application this is called ‘Projection on Planes’. This algorithm creates rotation matrix which rotates the reference points around the ‘z’ axis such that the ‘y’ values of the top-left and top-right reference points are equal. It then creates another rotation matrix which rotates these new points around the ‘y’ axis such that the ‘z’ values are equal. It then takes the output from that rotation and creates a transformation matrix such that the top-left reference now sits at (0,0) and the bottom-right sits at (1,1). Finally the current readings is passed through these through matrices giving ‘x’ and ‘y’ values each in the range of [0,1]. These values are then multiplied by the width and height of the screen respectively in order to get screen coordinates. 

**Selection of Various Pen Attributes**

The strength of the magnetic field created by the electromagnet can be used in order to set several pen attributes. Dialogs have been implemented in order to set the color and stroke size of the pen. In order to set these attributes the magnetic pen to first be calibrated. When the dialog is then opened the user can set the color or stroke width by moving the pen left and right. The dialogs have dynamic feedback to show the currently selected value and if the user confirms the selection the appropriate attribute will be set. Naturally, selecting the cancel option will cancel the dialog and keep the previously set attribute values. 
The dialogs simply use the magnitude of the magnetic field strength in order to set these values, with the minimum and maximum values represented by the minimum and maximum detected during calibration respectively. Any values outside of this range will simply be clamped.
